==> SomatiCA_1.6.0.tar.gz: SomatiCA/R/refineSegment.R <==
refineSegment <- function(segmentwithratio, data, threshold1 = 0.01, threshold2 = 0.05, adjust = FALSE, method = "mle"){
  if(class(segmentwithratio) != "GRanges" | class(data) != "GRanges")
    stop("Wrong input. Please check the manual.")  
  l <- i <- NULL
  suppressPackageStartupMessages(require(foreach))
  asegment <- as(segmentwithratio, "data.frame")
  x <- as(data, "data.frame")  
  tumor.med <- median(x$tCount)
  normal.med <- median(x$tCountN)
  if(adjust == FALSE) x$adjust.tCount <- x$tCount
  else x$adjust.tCount <- x$tCount*normal.med/tumor.med 
  remove.outlier <- function(data, k = 0.1 ){
    quant <- quantile(data, probs = c(k, 1-k))
    flag <- data > quant[1] & data < quant[2]
    return(c(1:length(data))[flag])
  }
  
  newseg <- foreach(l=unique(as.character(asegment$seqnames)), .combine=rbind) %do% {
    sub <- asegment[asegment$seqnames==l, ] 
    if(nrow(sub)==1) return(sub)
    flag1 <- min(abs(diff(sub$medLAF)))
    flag2 <- min(abs(diff(sub$ratio)))
    onechr_results <- sub
    while(flag1 < threshold1 | flag2 < threshold2){
      len <- length(onechr_results[, 1])
      m <- 1
      total <- NULL
      while(1){
        n <- 1
        while((abs(onechr_results$ratio[m]-onechr_results$ratio[m+n]) <= threshold2 | abs(onechr_results$medLAF[m]-onechr_results$medLAF[m+n]) <= threshold1)& m+n<=len){
          n <- n+1
        }
        group <- c(m, m+n-1)
        total <- rbind(total, group)
        m <- m+n
        if(m == len){ 
          group <- c(m, m)
          total <- rbind(total, group)
          break
        }
        else if(m > len){
          break
        }
      }
      new.start <- onechr_results$start[total[, 1]]
      new.end <- onechr_results$end[total[, 2]]
      onechr <- data.frame(l, new.start, new.end)
      colnames(onechr) <- c("seqnames", "start", "end")
      results <- foreach(i=1:length(onechr$seqnames), .combine=rbind) %do% {   
        mm  <- x[x$start < onechr$end[i] & x$start >= onechr$start[i] & as.character(x$seqnames) == as.character(onechr$seqnames[i]), ]
        if(nrow(mm)==0) stop("Segment and data do not match.")	
        if(nrow(mm)<5){
          ratio <- round(exp(sum(log(mm$tCount/mm$tCountN))/nrow(mm)), 3) 
        }
        else{
          a1 <- as.numeric(mm$adjust.tCount)
          a2 <- as.numeric(mm$tCountN)
          aa <- intersect(remove.outlier(a1), remove.outlier(a2))
          aR <- a1/a2	
          if(method == "mean") {
            suppressPackageStartupMessages(require(sn))
            if(length(a1[aa]) > 5){			
              est.tumor.mean <- sn.mle(y = a1[aa], plot.it=F)$cp[1]
              est.normal.mean <- sn.mle(y = a2[aa], plot.it=F)$cp[1]
            }
            else{
              est.tumor.mean <- mean(a1[aa])
              est.normal.mean <- mean(a2[aa])
            }
            somaticR <- round(est.tumor.mean/est.normal.mean, 3)	
          }
          tt <- aR[aa]
          fr <- function(l){
            sum((tt-l)^2/(tt^2+l))
          }
          mle.somaticR <- round(optimize(fr, c(0, 10))$minimum, 3)
          geometric.somaticR <- round(exp(sum(log(tt))/length(tt)), 3)  
          ratio <- switch(method, mle = mle.somaticR, mean = somaticR, 
                          geometric = geometric.somaticR)
        }
        if(ratio==10) ratio <- round(exp(sum(log(mm$tCount/mm$tCountN))/nrow(mm)), 3)       
        return(c(round(median(mm$LAF, na.rm=TRUE), 4), round(median(mm$germLAF, na.rm=TRUE), 4), ratio)) 			
      }    
      if(nrow(onechr)==1) onechr_results <- data.frame(onechr, results[1], results[2], results[3])
      else  onechr_results <- data.frame(onechr, results)
      colnames(onechr_results) <- c("seqnames", "start", "end", "medLAF", "medgLAF", "ratio")
      if(nrow(onechr_results)==1)  break 
      flag1 <- min(abs(diff(onechr_results$medLAF)))
      flag2 <- min(abs(diff(onechr_results$ratio)))
      if(flag1 > threshold1 & flag2 > threshold2) break
    }  
    return(onechr_results)
  } 
  refinedwithratio <- GRanges(seqnames=newseg$seqnames, 
                       ranges=IRanges(start=newseg$start, end=newseg$end),
                       medLAF=newseg$medLAF,
                       medgLAF=newseg$medgLAF,
                       ratio=newseg$ratio) 
  return(refinedwithratio)            
}

==> SomatiCA_1.12.0.tar.gz: SomatiCA/R/refineSegment.R <==
refineSegment <- function(segmentwithratio, data, threshold1 = 0.01, threshold2 = 0.05, adjust = FALSE, method = "mle"){
  if(class(segmentwithratio) != "GRanges" | class(data) != "GRanges")
    stop("Wrong input. Please check the manual.")  
  l <- i <- NULL
  suppressPackageStartupMessages(require(foreach))
  asegment <- as(segmentwithratio, "data.frame")
  x <- as(data, "data.frame")  
  tumor.med <- median(x$tCount)
  normal.med <- median(x$tCountN)
  if(adjust == FALSE) {
    x$adjust.tCount <- x$tCount
  } else {
    x$adjust.tCount <- x$tCount*normal.med/tumor.med 
  }
  remove.outlier <- function(data, k = 0.1 ){
    quant <- quantile(data, probs = c(k, 1-k))
    flag <- data > quant[1] & data < quant[2]
    return(c(1:length(data))[flag])
  }
  
  newseg <- foreach(l=unique(as.character(asegment$seqnames)), .combine=rbind) %do% {
    sub <- asegment[asegment$seqnames==l, c("seqnames", "start", "end", "medLAF", "medgLAF", "ratio")] 
    if(nrow(sub)==1){ 
      mm  <- x[as.character(x$seqnames) == as.character(sub$seqnames), ]  
      a1 <- as.numeric(mm$adjust.tCount)
      a2 <- as.numeric(mm$tCountN)
      aa <- intersect(remove.outlier(a1), remove.outlier(a2))
      aR <- a1/a2	
      if(method == "mean") {
        suppressPackageStartupMessages(require(sn))
        if(length(a1[aa]) > 5){			
          est.tumor.mean <- sn.mle(y = a1[aa], plot.it=F)$cp[1]
          est.normal.mean <- sn.mle(y = a2[aa], plot.it=F)$cp[1]
        }
        else{
          est.tumor.mean <- mean(a1[aa])
          est.normal.mean <- mean(a2[aa])
        }
        somaticR <- round(est.tumor.mean/est.normal.mean, 3)	
      }
      tt <- aR[aa]
      fr <- function(l){
        sum((tt-l)^2/(tt^2+l))
      }
      mle.somaticR <- round(optimize(fr, c(0, 10))$minimum, 3)
      geometric.somaticR <- round(exp(sum(log(tt))/length(tt)), 3)  
      sub$ratio <- switch(method, mle = mle.somaticR, mean = somaticR, 
                          geometric = geometric.somaticR)
      return(sub)
    }
    else{
      flag1 <- min(abs(diff(sub$medLAF)))
      flag2 <- min(abs(diff(sub$ratio)))
      onechr_results <- sub
      while(flag1 < threshold1 | flag2 < threshold2){
        len <- length(onechr_results[, 1])
        m <- 1
        total <- NULL
        while(1){
          n <- 1
          while((abs(onechr_results$ratio[m]-onechr_results$ratio[m+n]) <= threshold2 | abs(onechr_results$medLAF[m]-onechr_results$medLAF[m+n]) <= threshold1)& m+n<=len){
            n <- n+1
          }
          group <- c(m, m+n-1)
          total <- rbind(total, group)
          m <- m+n
          if(m == len){ 
            group <- c(m, m)
            total <- rbind(total, group)
            break
          }
          else if(m > len){
            break
          }
        }
      new.start <- onechr_results$start[total[, 1]]
      new.end <- onechr_results$end[total[, 2]]
      onechr <- data.frame(l, new.start, new.end)
      colnames(onechr) <- c("seqnames", "start", "end")
      results <- foreach(i=1:length(onechr$seqnames), .combine=rbind) %do% {   
        mm  <- x[x$start < onechr$end[i] & x$start >= onechr$start[i] & as.character(x$seqnames) == as.character(onechr$seqnames[i]), ]
        if(nrow(mm)==0) stop("Segment and data do not match.")	
        if(nrow(mm)<5){
          ratio <- round(exp(sum(log(mm$tCount/mm$tCountN))/nrow(mm)), 3) 
        }
        else{
          a1 <- as.numeric(mm$adjust.tCount)
          a2 <- as.numeric(mm$tCountN)
          aa <- intersect(remove.outlier(a1), remove.outlier(a2))
          aR <- a1/a2	
          if(method == "mean") {
            suppressPackageStartupMessages(require(sn))
            if(length(a1[aa]) > 5){			
              est.tumor.mean <- sn.mle(y = a1[aa], plot.it=F)$cp[1]
              est.normal.mean <- sn.mle(y = a2[aa], plot.it=F)$cp[1]
            }
            else{
              est.tumor.mean <- mean(a1[aa])
              est.normal.mean <- mean(a2[aa])
            }
            somaticR <- round(est.tumor.mean/est.normal.mean, 3)	
          }
          tt <- aR[aa]
          fr <- function(l){
            sum((tt-l)^2/(tt^2+l))
          }
          mle.somaticR <- round(optimize(fr, c(0, 10))$minimum, 3)
          geometric.somaticR <- round(exp(sum(log(tt))/length(tt)), 3)  
          ratio <- switch(method, mle = mle.somaticR, mean = somaticR, 
                          geometric = geometric.somaticR)
        }
        if(ratio==10) ratio <- round(exp(sum(log(mm$tCount/mm$tCountN))/nrow(mm)), 3)       
        return(c(round(median(mm$LAF[grep("het", mm$zygosity)], na.rm=TRUE), 4), round(median(mm$germLAF[grep("het", mm$zygosity)], na.rm=TRUE), 4), ratio)) 			
      }    
      if(nrow(onechr)==1) onechr_results <- data.frame(onechr, results[1], results[2], results[3])
      else  onechr_results <- data.frame(onechr, results)
      colnames(onechr_results) <- c("seqnames", "start", "end", "medLAF", "medgLAF", "ratio")
      if(nrow(onechr_results)==1)  break 
      flag1 <- min(abs(diff(onechr_results$medLAF)))
      flag2 <- min(abs(diff(onechr_results$ratio)))
      if(flag1 > threshold1 & flag2 > threshold2) break
    }  
    return(onechr_results)
    }
  } 
  refinedwithratio <- GRanges(seqnames=newseg$seqnames, 
                       ranges=IRanges(start=newseg$start, end=newseg$end),
                       medLAF=newseg$medLAF,
                       medgLAF=newseg$medgLAF,
                       ratio=newseg$ratio) 
  return(refinedwithratio)            
}
